﻿/*
 * Copyright(c) 2009 cuegraphix.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package com.cuegraphix.layouts
{
	import com.cuegraphix.events.LiquidLayoutEvent;
	import flash.errors.IllegalOperationError;
	import flash.utils.Dictionary;
	
	/**
	* レイアウトに登録されるターゲットの制御情報クラス。
	* @author cue
	*/
	public class LiquidLayoutListener
	{
		/**
		 * 変換するターゲットを取得します。
		 */
		//public function get target():Object { return _target; }
		//private var _target:Object;
		
		private var _properties:Dictionary;
		
		public var eventType:String = LiquidLayoutEvent.RESIZE_ALWAYS;
		public var enabled:Boolean;
		
		public var onResize:Function;
		public var onResizeStart:Function;
		public var onResizeProgress:Function;
		public var onResizeComplete:Function;
		public var onResizeParams:Array;
		public var onResizeStartParams:Array;
		public var onResizeProgressParams:Array;
		public var onResizeCompleteParams:Array;
		
		public function LiquidLayoutListener(  )
		{
			_properties = new Dictionary( true );
		}
		
		public function addProperty( prop:LiquidLayoutProperty ):void
		{
			_properties[prop.property] = prop;
		}
		
		public function getPropertyByAlign( align:String ):LiquidLayoutProperty
		{
			for each ( var prop:LiquidLayoutProperty in _properties )
			{
				if ( prop.align == align ) { return prop; }
			}
			return null;
		}
		
		public function getPropertyByPropertyName( propName:String ):LiquidLayoutProperty
		{
			for each ( var prop:LiquidLayoutProperty in _properties )
			{
				if ( prop.property == propName ) { return prop; }
			}
			return null;
		}
		
		public function removePropertyByAlign( align:String ):void
		{
			removeProperty( getPropertyByAlign( align ) );
		}
		
		public function removePropertyByPropertyName( propName:String ):void
		{
			removeProperty( getPropertyByPropertyName( propName ) );
		}
		
		public function removeProperty( prop:LiquidLayoutProperty ):void
		{
			delete _properties[prop];
		}
		
		public function removeAllProperties():void
		{
			for each ( var prop:LiquidLayoutProperty in _properties )
			{
				removeProperty( prop );
			}
		}
		
		public function update( layout:LiquidLayout, target:Object, nowEventType:String = null ):void
		{
			if ( !enabled ) { return; }
			if ( nowEventType == null || eventType == nowEventType )
			{
				for each ( var prop:LiquidLayoutProperty in _properties )
				{
					prop.transform( layout, target );
				}
			}
			
			if ( ( nowEventType == null || nowEventType == LiquidLayoutEvent.RESIZE_START ) && onResizeStart != null )
			{
				try
				{
					onResizeStart.apply( target, onResizeStartParams );
				}
				catch ( e:Error )
				{
					throw new IllegalOperationError( formatErrorMessage( eventType ) );
				}
			}
			else if ( ( nowEventType == null || nowEventType == LiquidLayoutEvent.RESIZE_PROGRESS ) && onResizeProgress != null )
			{
				try
				{
					onResizeProgress.apply( target, onResizeProgressParams );
				}
				catch ( e:Error )
				{
					throw new IllegalOperationError( formatErrorMessage( eventType ) );
				}
			}
			else if ( ( nowEventType == null || nowEventType == LiquidLayoutEvent.RESIZE_COMPLETE ) && onResizeComplete != null )
			{
				try
				{
					onResizeComplete.apply( target, onResizeCompleteParams );
				}
				catch ( e:Error )
				{
					throw new IllegalOperationError( formatErrorMessage( eventType ) );
				}
			}
			if ( onResize != null )
			{
				try
				{
					onResize.apply( target, onResizeParams );
				}
				catch ( e:Error )
				{
					throw new IllegalOperationError( formatErrorMessage( eventType ) );
				}
			}
		}
		
		protected function formatErrorMessage( eventType:String ):String
		{
			return eventType + " メソッド実行中にエラーが発生しました。";
		}
		
		public function destroy():void
		{
			removeAllProperties();
			_properties = null;
			eventType = null;
			enabled = false;
			onResize = null;
			onResizeStart = null;
			onResizeProgress = null;
			onResizeComplete = null;
			onResizeParams = null;
			onResizeStartParams = null;
			onResizeProgressParams = null;
			onResizeCompleteParams = null;
		}
		
	}
	
}