﻿/*
 * Copyright(c) 2009 cuegraphix.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package com.cuegraphix.layouts
{
	
	/**
	* レイアウト制御の変更プロパティ、変更差分、実行関数を持つクラス
	* @author cue
	*/
	public class LiquidLayoutProperty
	{
		public var align:String;
		public var value:Object;
		public var property:String;
		public var extra:Object;
		
		/**
		 * レイアウト制御のプロパティを登録します。
		 * @param	align		layoutオブジェクト対応プロパティ名
		 * @param	value		変更差分情報
		 * @param	property	変更するプロパティ名
		 * @param	extra		その他使用情報
		 */
		public function LiquidLayoutProperty( align:String, value:Object, property:String, extra:Object = null )
		{
			this.align = align;
			this.value = value;
			this.property = property;
			this.extra = extra;
		}
		
		/**
		 * レイアウト制御を実行した値をターゲットプロパティに代入します。
		 * @param	layout	LiquidLayoutオブジェクト
		 * @param	target	プロパティの値を代入するターゲット
		 */
		public function transform( layout:LiquidLayout, target:Object ):void
		{
			var trans:Object;
			
			if ( value is String )
			{
				var piv:Number = parseInt( String( value ) );
				if ( !isNaN( piv ) )
				{
					if ( layout.calcParcentProperties[align] )
					{
						trans = layout[align] + ( layout[layout.calcParcentProperties[align]] * ( piv / 100 ) );
					}
					else
					{
						trans = layout[align] * ( piv / 100 );
					}
				}
			}
			else if ( value is Function )
			{
				trans = value.apply( target, [ layout[align] ] );
			}
			else if ( value is Number )
			{
				trans = layout[align] + value;
			}
			
			target[ property ] = trans;
		}
		
		public function toString():String
		{
			return "[LiquidLayoutProperty align:" + align + " property:" + property + "]";
		}
	}
	
}