﻿/*
 * Copyright(c) 2009 cuegraphix.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

 package com.cuegraphix.layouts.properties 
{
	import com.cuegraphix.layouts.LiquidLayout;
	import com.cuegraphix.layouts.LiquidLayoutProperty;
	import flash.display.DisplayObject;
	
	/**
	* 背景画像や写真などを比率を固定したまま画面いっぱいに表示させるための LiquidLayoutProperty 拡張クラス。
	* 拡大写真などで画像全体を画面にfixさせるinsideと、背景画像など余白なく(画像最小比率を画面最大比率に合わせ)表示させるoutsideプロパティを設定できます。
	* @author cue
	*/
	public class ConstrainProportions extends LiquidLayoutProperty
	{
		public static const FIX_INSIDE:String  = "inside";
		public static const FIX_OUTSIDE:String = "outside";
		
		/**
		 * @see com.cuegraphix.layouts.LiquidLayoutProperty
		 */
		public function ConstrainProportions( align:String, value:Object, property:String, extra:Object = null )
		{
			super( align, value, property, extra );
		}
		
		/**
		 * スペシャルプロパティを使用するために、LiquidLayout に登録します。
		 * このメソッドはプロパティを使用する前に呼び出します。
		 */
		public static function init():void
		{
			LiquidLayout.registSpecialProperties( "inside", ConstrainProportions );
			LiquidLayout.registSpecialProperties( "outside", ConstrainProportions );
		}
		
		/**
		 * @see com.cuegraphix.layouts.LiquidLayoutProperty#transform
		 */
		public override function transform( layout:LiquidLayout, target:Object ):void
		{
			var d:DisplayObject = target as DisplayObject;
			if (d == null) return;
			
			var defaultWidth:Number  = d.width  / d.scaleX;
			var defaultHeight:Number = d.height / d.scaleY;
			var scaleWidth:Number    = layout.width;
			var scaleHeight:Number   = layout.height;
			if (value is Array)
			{
				if (!isNaN(value[0])) scaleWidth  += value[0];
				if (!isNaN(value[1])) scaleHeight += value[1];
			}
			else if (value is Number)
			{
				scaleWidth  += value;
				scaleHeight += value;
			}
			else if (value is Object)
			{
				if (!isNaN(value.width))  scaleWidth  += value.width;
				if (!isNaN(value.height)) scaleHeight += value.height;
			}
			
			var h:Number = defaultWidth  / scaleWidth;
			var v:Number = defaultHeight / scaleHeight;
			var r:Number = defaultWidth  / defaultHeight;
			
			var size:Object;
			if (align == FIX_INSIDE)
			{
				size = getInsideSize(h, v, r, scaleWidth, scaleHeight);
			}
			else if (align == FIX_OUTSIDE)
			{
				size = getOutsideSize(h, v, r, scaleWidth, scaleHeight);
			}
			else
			{
				return;
			}
			
			d.width  = size.width;
			d.height = size.height;
			d.x      = Math.round(layout.center - (size.width / 2));
			d.y      = Math.round(layout.middle - (size.height / 2));
		}
		
		private function getInsideSize(horizontalPercent:Number, verticalPercent:Number, aspectRatio:Number, width:Number, height:Number)
		{
			var w:Number;
			var h:Number;
			if (horizontalPercent > verticalPercent)
			{
				w = width;
				h = Math.round(width / aspectRatio);
			}
			else if(horizontalPercent < verticalPercent)
			{
				w = Math.round(height * aspectRatio);
				h = height;
			}
			else
			{
				w = width;
				h = height;
			}
			return { width: w, height: h };
		}
		
		private function getOutsideSize(horizontalPercent:Number, verticalPercent:Number, aspectRatio:Number, width:Number, height:Number)
		{
			var w:Number;
			var h:Number;
			if (horizontalPercent < verticalPercent)
			{
				w = width;
				h = Math.round(width / aspectRatio);
			}
			else if(horizontalPercent > verticalPercent)
			{
				w = Math.round(height * aspectRatio);
				h = height;
			}
			else
			{
				w = width;
				h = height;
			}
			return { width: w, height: h };
		}
	}
	
}